--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'Bison'
local Addon = LibStub( 'AceAddon-3.0'):NewAddon( MAJOR, 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0')
if not Addon then return end -- already loaded and no upgrade necessary

local L      = LibStub( 'AceLocale-3.0'):GetLocale( MAJOR)
local AceGUI = LibStub( 'AceGUI-3.0')

Addon.name, Addon.localizedname = GetAddOnInfo( MAJOR)
Addon.version = format( '%s: %s |cFF20FF20(Cata)|r', L.Version, GetAddOnMetadata( MAJOR, 'Version') or 'unknown')

local DEFAULTS = { 
	profile = {
		['*']  = { xPos = 400, yPos = 300, xPadding = 5, yPadding = 15, scale = 1.0, rows = 1, cols = 16, number = 16, gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'dec',   style = {}, },
		buff   = { xPos = 400, yPos = 500, xPadding = 5, yPadding = 15, scale = 1.0, rows = 2, cols = 20, number = 40, gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'dec',   style = {}, },
		secure = { xPos = 400, yPos = 300, xPadding = 5, yPadding = 15, scale = 1.0, rows = 2, cols = 20, number = 40, gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'dec',   style = {}, },
		prog   = { xPos = 400, yPos = 150, xPadding = 5, yPadding = 15, scale = 1.0, rows = 1, cols = 16, number = 16, gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'alpha', style = {}, },
		hidden = { xPos = 400, yPos = 100, xPadding = 5, yPadding = 15, scale = 1.0, rows = 2, cols =  8, number = 16, gap = 0, show = true, timer = true, flashing = false, horizontal = true, sort = 'dec',   style = {}, },
		debuff = { xPos = 400, yPos = 250, xPadding = 5, yPadding = 15, scale = 1.0, rows = 1, cols = 16, number = 16, gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'dec',   style = {}, },
		weapon = { xPos = 400, yPos = 200, xPadding = 5, yPadding = 15, scale = 1.0, rows = 1, cols = 2,  number = 2,  gap = 0, show = true, timer = true, flashing = true,  horizontal = true, sort = 'none',  style = {}, },
		join     = false,
		blizzard = false,
		locked   = true,
		enabled  = true,
	} 
}
local UPDATE_TIME = 0.2
local NOTHING = {}

local frame = CreateFrame( 'Frame', 'BisonFrame')
local groups = {}
local update = UPDATE_TIME
local auras = {
	buff = {},
	debuff = {},
	weapon = {},
	consolidation = 0
}

--[[---------------------------------------------------------------------------------
Local
------------------------------------------------------------------------------------]]
local function InitBuffs()
	for i = 1,40 do
		tinsert( auras.buff, {})
	end
	for i = 1,16 do
		tinsert( auras.debuff, {})
	end
	for i = 1,2 do
		tinsert( auras.weapon, {})
	end
end

local function UpdateConsolitation( aura)
	aura.hide = aura.consolidate and aura.timeLeft and aura.duration > 30
	if aura.hide then
		auras.consolidation = auras.consolidation + 1
	end
end

local function UpdateTime( aura, elapsed)
	aura.timeLeft = aura.timeLeft and max( aura.timeLeft - elapsed, 0)
	aura.warn = aura.timeLeft and (aura.timeLeft < BUFF_DURATION_WARNING_TIME) or nil
end

local function UpdateModules()
	if #groups > 0 then
		local name = tremove( groups, 1)
		tinsert( groups, name)
		local mod = Addon:GetModule( name, true)
		if mod and mod.UpdateValues then
			mod:UpdateValues()
		end
	end
end

local function Frame_OnUpdate( frame, elapsed)
	auras.consolidation = 0
	for _,aura in pairs( auras.buff) do
		UpdateTime( aura, elapsed)
		UpdateConsolitation( aura)
	end
	for _,aura in pairs( auras.debuff) do
		UpdateTime( aura, elapsed)
	end
	for _,aura in pairs( auras.weapon) do
		UpdateTime( aura, elapsed)
	end
	if update < 0 then
		local n = #groups
		update = (n > 0) and (UPDATE_TIME / n) or UPDATE_TIME
		UpdateModules()
	else
		update = update - elapsed
	end
end

local function UpdateAura( id, aura, filter, now)
	name, rank, texture, count, debuffType, duration, expiration, unitCaster, isStealable, consolidate, spellID = UnitAura( PlayerFrame.unit, id, filter)
	local timeLeft = name and expiration and (expiration - now) or nil
	aura.id      = id
	aura.filter  = filter
	aura.name    = name
	aura.rank    = rank
	aura.texture = texture
	aura.count   = count
	aura.type    = debuffType
	aura.duration    = duration
	aura.expiration  = expiration
	aura.timeLeft    = timeLeft and (timeLeft > 0) and timeLeft or nil
	aura.duration    = duration
	aura.unitCaster  = unitCaster
	aura.isStealable = isStealable
	aura.consolidate = consolidate
	aura.spellID     = spellID
end

local function Frame_OnAura()
	local now = GetTime()
	auras.consolidation = 0
	for id,aura in pairs( auras.buff) do
		UpdateAura( id, aura, 'HELPFUL', now)
		UpdateConsolitation( aura)
	end
	for id,aura in pairs( auras.debuff) do
		UpdateAura( id, aura, 'HARMFUL', now)
	end
	local mainEnchant, mainTimeLeft, mainCharges, offEnchant, offTimeLeft, offCharges = GetWeaponEnchantInfo()
	mainTimeLeft = mainEnchant and mainExpiration and (mainExpiration / 1000)
	offTimeLeft  = offEnchant and offExpiration and (offExpiration / 1000)
	local auraMH = auras.weapon[1]
	auraMH.id       = 16
	auraMH.name     = mainEnchant 
	auraMH.count    = mainCharges 
	auraMH.texture  = mainEnchant and GetInventoryItemTexture( PlayerFrame.unit, 16) 
	auraMH.timeLeft = mainTimeLeft and (mainTimeLeft > 0) and mainTimeLeft or nil
	local auraOH = auras.weapon[2]
	auraOH.id       = 17
	auraOH.name     = offEnchant 
	auraOH.count    = offCharges 
	auraOH.texture  = offEnchant and GetInventoryItemTexture( PlayerFrame.unit, 17) 
	auraOH.timeLeft = offTimeLeft and (offTimeLeft > 0) and offTimeLeft or nil
end

local oldConsolidate
local function SetBlizzardVisible( value)
	if value ~= BuffFrame:IsVisible() then
		if value then
--			BlizzardOptionsPanel_SetCVarSafe( 'consolidateBuffs', oldConsolidate)
--			CONSOLIDATE_BUFFS = oldConsolidate
--			_G['CONSOLIDATE_BUFFS'] = oldConsolidate
			BuffFrame:Show()
			TemporaryEnchantFrame:Show()
		else
--			oldConsolidate = _G['CONSOLIDATE_BUFFS']
--			BlizzardOptionsPanel_SetCVarSafe( 'consolidateBuffs', '0')
--			CONSOLIDATE_BUFFS = '0'
--			_G['CONSOLIDATE_BUFFS'] = '0'
			BuffFrame:Hide()
			TemporaryEnchantFrame:Hide()
			ConsolidatedBuffs:Hide()
		end
		BuffFrame_Update()
	end
end

--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Addon:OnInitialize()
	InitBuffs()
	self.db = LibStub( 'AceDB-3.0'):New( 'BisonDB', DEFAULTS, 'Default')
	self:SetEnabledState( self.db.profile.enabled)
	self.db.RegisterCallback( self, 'OnNewProfile')
	self.db.RegisterCallback( self, 'OnProfileChanged')
	self.db.RegisterCallback( self, 'OnProfileCopied')
	self.db.RegisterCallback( self, 'OnProfileReset')
	self.db.RegisterCallback( self, 'OnProfileDeleted')
end

function Addon:OnEnable()
	self:Debug( 'OnEnable')
	self:ToggleDebugLog( false)
	SetBlizzardVisible( self.db.profile.blizzard)
	frame:SetScript( 'OnUpdate', Frame_OnUpdate)
	Frame_OnAura()
	self:RegisterEvent( 'UNIT_AURA')
	self:RegisterMessage( 'BISON_DEBUG')
	self:Print( self.version, 'loaded')
end

function Addon:OnDisable()
	self:Debug( 'OnDisable')
	frame:SetScript( 'OnUpdate', nil)
	self:UnregisterEvent( 'UNIT_AURA')
	self:UnregisterMessage( 'BISON_DEBUG')
	SetBlizzardVisible( self.db.profile.blizzard)
end

function Addon:UNIT_AURA( event, unit)
	if unit == PlayerFrame.unit then
		Frame_OnAura()
		self:SendMessage( 'BISON_UPDATE')
	end
end

function Addon:BISON_DEBUG( event, debug)
	self:ToggleDebugLog( debug)
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Addon:ShowBlizzard( value)
	self.db.profile.blizzard = value
	SetBlizzardVisible( value)
end

function Addon:CloneAura( dst, name)
	for _,aura in pairs( auras[name] or NOTHING) do
		tinsert( dst, aura)
	end 
end

function Addon:GetConsolidation()
	local profile = self.db.profile
	return profile.join and profile.locked and auras.consolidation or 0
end

function Addon:AddModule( name)
	tinsert( groups, name)
end

function Addon:RemoveModule( name)
	local index = 1
	while groups[index] do
		if name == groups[index] then
			tremove( groups, index)
		else
			index = index + 1
		end
	end
end

--[[---------------------------------------------------------------------------------
Profile
------------------------------------------------------------------------------------]]
function Addon:ListProfiles()
	self:Print( L.ProfileAvailable)
	local current = self.db:GetCurrentProfile()
	for _,name in ipairs(self.db:GetProfiles()) do
		self:Print( format( (name == current) and ' - |cFFFFFF00%s|r' or ' - %s', name))
	end
end

function Addon:OnNewProfile( event, db, name)
	self:SendMessage( 'BISON_UPDATE')
	self:Print( L.ProfileCreated:format( name))
end

function Addon:OnProfileChanged( event, db, name)
	self:SendMessage( 'BISON_UPDATE')
	self:Print( L.ProfileLoaded:format( name))
end

function Addon:OnProfileCopied( event, db, name)
	self:SendMessage( 'BISON_UPDATE')
	self:Print( L.ProfileCopied:format( name))
end

function Addon:OnProfileReset( event, db)
	self:SendMessage( 'BISON_UPDATE')
	self:Print( L.ProfileReset:format( db:GetCurrentProfile()))
end

function Addon:OnProfileDeleted( event, db, name)
	self:Print( L.ProfileDeleted:format( name))
end

--[[---------------------------------------------------------------------------------
Layout
------------------------------------------------------------------------------------]]
AceGUI:RegisterLayout( 'BuffBar', function( content, children)
	local profile = Addon.db.profile[content.obj.moduleName]
	local xPadding = -profile.xPadding or 0
	local yPadding = -profile.yPadding or 0
	local horizontal = profile.horizontal
	local scale = profile.scale or 1
	local cols = profile.cols or 10
	local rows = profile.rows or 4
	local last, first
	local width = 0
	local height = 0
	local i = 1
	for k,child in pairs( children) do
		local frame = child.frame
		frame:ClearAllPoints()
		frame:SetScale( scale)
		if last and first then
			if horizontal then
				if xPadding >= 0 then
					frame:SetPoint( 'LEFT', last, 'RIGHT', xPadding, 0)
				else
					frame:SetPoint( 'RIGHT', last, 'LEFT', xPadding, 0)
				end
			else
				if yPadding >= 0 then
					frame:SetPoint( 'BOTTOM', last, 'TOP', 0, yPadding)
				else
					frame:SetPoint( 'TOP', last, 'BOTTOM', 0, yPadding)
				end
			end
		elseif first then
			if horizontal then
				if yPadding >= 0 then
					frame:SetPoint( 'BOTTOM', first, 'TOP', 0, yPadding)
				else
					frame:SetPoint( 'TOP', first, 'BOTTOM', 0, yPadding)
				end
			else
				if xPadding >= 0 then
					frame:SetPoint( 'LEFT', first, 'RIGHT', xPadding, 0)
				else
					frame:SetPoint( 'RIGHT', first, 'LEFT', xPadding, 0)
				end
			end
			first = frame
		else
			if xPadding >= 0 then
				if yPadding >= 0 then
					frame:SetPoint( 'BOTTOMLEFT', content)
				else
					frame:SetPoint( 'TOPLEFT', content)
				end
			else
				if yPadding >= 0 then
					frame:SetPoint( 'BOTTOMRIGHT', content)
				else
					frame:SetPoint( 'TOPRIGHT', content)
				end
			end
			first = frame
		end
		last = frame
		if child.DoLayout then
			child:DoLayout()
		end
		width  = max( width, frame.width or frame:GetWidth() or 0)
		height = max( height, frame.height or frame:GetHeight() or 0)
		i = i + 1
		if horizontal then
			if i > cols then
				i = 1
				last = nil
			end
		else
			if i > rows then
				i = 1
				last = nil
			end
		end
	end
	content.obj:LayoutFinished( scale * cols * (width + abs(xPadding)), scale * rows * (height + abs(yPadding)))
end)
