--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Type, Version = 'BisonSecure', 1
local AceGUI = LibStub( 'AceGUI-3.0')

UIFrameFlashStop = UIFrameFlashRemoveFrame or UIFrameFlashStop

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnMouseDown( frame)
	frame.obj:Fire( 'OnMouseDown')
end

local function Control_OnMouseUp( frame)
	frame.obj:Fire( 'OnMouseUp')
end

local function Control_OnEnter( frame)
	frame.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( frame)
	frame.obj:Fire( 'OnLeave')
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	['OnAcquire'] = function( self)
		self:SetGap( 0)
	end,

	['OnRelease'] = function( self)
		self:SetFlashing( false)
	end,

	['SetText'] = function( self, text, warn)
		if text then
			self.left:SetText( text)
			if warn then
				self.left:SetVertexColor( HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			else
				self.left:SetVertexColor( NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			end
			self:Fire( 'OnTooltip')
		else
			self.left:SetText( ' ')
			self.left:SetVertexColor( NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		end
	end,
	
	['SetCount'] = function( self, count)
		if count and count > 1 then
			self.count:SetText( count)
			self.count:Show()
		else
			self.count:Hide()
		end
	end,

	['SetSpell'] = function( self, id)
		if id then
			self.spell:SetText( id)
			self.spell:Show()
		else
			self.spell:Hide()
		end
	end,

	['SetSymbol'] = function( self, text)
		if text then
			self.symbol:SetText( text)
			self.symbol:Show()
		else
			self.symbol:Hide()
		end
	end,

	['SetTexture'] = function( self, texture, ...)
		if texture then
			self.icon:SetTexture( texture)
			local n = select( '#', ...)
			if n == 4 or n == 8 then
				self.icon:SetTexCoord( ...)
			else
				self.icon:SetTexCoord( 0, 1, 0, 1)
			end
			self.icon:Show()
		else
			self.icon:Hide()
		end
	end,

	['SetBorder'] = function( self, color)
		if color then
			self.border:SetVertexColor( color.r, color.g, color.b, 1)
			self.border:Show()
		else
			self.border:SetVertexColor( 0, 0, 0, 0)
			self.border:Hide()
		end
	end,

	['SetSpell'] = function( self, filter, spell)
		local frame = self.frame
		frame:SetAttribute( 'type', 'cancelaura')
		frame:SetAttribute( 'unit', 'player')
		if filter then
			frame:SetAttribute( 'index', spell)
		else
			frame:SetAttribute( 'index', nil)
			frame:SetAttribute( 'spell', spell)
		end
	end,

	['SetFlashing'] = function( self, shouldFlash)
		if shouldFlash then
			if not UIFrameIsFading( self.frame) then
				UIFrameFlash( self.frame, 0.25, 0.25, BUFF_WARNING_TIME, true, 0.25, 0.25) 
			end
		else
			UIFrameFlashRemoveFrame( self.frame)
			self.frame:SetAlpha( 1)
		end
	end,

	['SetLock'] = function( self, locked)
		if locked then
			self.frame:SetScript( 'OnMouseDown', nil)
			self.frame:SetScript( 'OnMouseUp',   nil)
			if not self.userdata.name then
				self.frame:Hide()
			end
			self.ghost:Hide()
			self.back:Hide()
		else
			self.frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
			self.frame:SetScript( 'OnMouseUp',   Control_OnMouseUp)
			self.frame:Show()
			self.ghost:Show()
			self.back:Show()
		end
	end,

	['SetCooldown'] = function( self, start, duration, enabled)
		local cooldown = self.cooldown
		if enabled then
			cooldown:SetCooldown( start, duration)
			cooldown:Show()
		else
			cooldown:Hide()
		end
	end,

	['SetGap'] = function( self, gap)
		self.userdata.gap = tonumber( gap) or 0
		self.left:ClearAllPoints()
		self.left:SetPoint( 'TOP', self.frame, 'BOTTOM', 0, -self.userdata.gap)
	end,
	
	['Initialize'] = function( self, index)
		self.ghost:SetText( index)
	end,

	['SetVisible'] = function( self, visible)
		if visible then
			self.frame:Show()
		else
			self:SetFlashing( false)
			self.frame:Hide()
			if GameTooltip:IsOwned( self.frame) then
				GameTooltip:Hide()
			end
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)
	local name = Type .. id
	
	local frame = CreateFrame( 'Button', name, UIParent, 'SecureActionButtonTemplate')
	frame:SetAlpha( 1)
	frame:SetWidth( 30)
	frame:SetHeight( 30)
	frame:EnableMouse( true)
	frame:RegisterForClicks( 'RightButtonUp')
	frame:SetScript( 'OnEnter', Control_OnEnter)
	frame:SetScript( 'OnLeave', Control_OnLeave)
	frame:Hide()
	
	local cooldown = CreateFrame( 'Cooldown', name..'Cooldown', frame)
	cooldown:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	cooldown:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	cooldown:Hide()
	
	local icon = frame:CreateTexture( name..'Icon', 'BACKGROUND')
	icon:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	icon:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	
	local left = frame:CreateFontString( name..'Name', 'ARTWORK', 'GameFontNormalSmall')
	left:SetPoint( 'TOP', frame, 'BOTTOM', 0, 0)

	local spell = frame:CreateFontString( name..'Spell', 'BACKGROUND', 'GameFontWhiteSmall')
	spell:SetPoint( 'TOP', left, 'BOTTOM', 0, 0)
	spell:Hide()

	local count = frame:CreateFontString( name..'Count', 'BACKGROUND', 'NumberFontNormal')
	count:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', -2, 2)
	count:Hide()

	local symbol = frame:CreateFontString( name..'Symbol', 'BACKGROUND', 'NumberFontNormal')
	symbol:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 2, -2)
	symbol:Hide()
	
	local border = frame:CreateTexture( name..'Border', 'OVERLAY')
	border:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	border:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	border:Hide()

	local ghost = frame:CreateFontString( nil, 'OVERLAY', 'GameFontHighlight')
	ghost:SetHeight( 10)
	ghost:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	ghost:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	ghost:Hide()

	local back = frame:CreateTexture( nil, 'OVERLAY')
	back:SetTexture( 0, 0, 1, 1)
	back:SetBlendMode( 'ADD')
	back:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	back:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	back:Hide()

	local widget = {
		num      = id,
		frame    = frame,
		cooldown = cooldown,
		icon     = icon,
		spell    = spell,
		count    = count,
		left     = left,
		symbol   = symbol,
		border   = border,
		ghost    = ghost,
		back     = back,
		type     = Type
	}
	for method, func in pairs( methods) do
		widget[method] = func
	end
	frame.obj = widget
	
	return AceGUI:RegisterAsWidget( widget)
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
