--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Addon = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L     = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')

local Module = Addon:NewModule( 'Options', 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0')

local SUB_HELP_MSG = ' - |cFF33FF99%s|r: %s'

--[[---------------------------------------------------------------------------------
Local
------------------------------------------------------------------------------------]]
local function OpenToCategory( appName, ...)
	local key = appName
	for n = 1,select( '#', ...) do
		key = key..'\001'..select( n, ...)
	end
	local grp = LibStub( 'AceConfigDialog-3.0').BlizOptions[appName]
	if grp and grp[key] then
		InterfaceOptionsFrame_OpenToCategory( grp[key].frame)
	end
end

local function OnCommand( input)
	local addonName = Addon:GetName()
	if input and input:trim() ~= '' then
		LibStub( 'AceConfigCmd-3.0'):HandleCommand( 'bison', addonName, input)
	else
		OpenToCategory( addonName, 'profile')
		OpenToCategory( addonName, 'main')
	end
end

local function SetEnable( info, value) 
	if value ~= Addon:IsEnabled() then
		if value then
			Addon:Enable()
			Module:SendMessage( 'BISON_UPDATE')
		else
			Addon:Disable()
		end
	end
end

local function SetDebug( info, value) 
	Module:SendMessage( 'BISON_DEBUG', value)
end

local function SetBlizzard( info, value) 
	Addon:ShowBlizzard( value)
end

function PrintVersion() 
	Addon:Print( Addon.version) 
end

function PrintHelp()
	local function PrintCmd( cmd, desc) Addon:Print( SUB_HELP_MSG:format( cmd, desc)) end
	Addon:Print( 'Commands (/bi, /bison)')
	PrintCmd( 'enabled', L.EnabledDesc)
	PrintCmd( 'debug', L.DebugDesc)
	PrintCmd( 'blizzard', L.BlizzDesc)
	PrintCmd( 'locked', L.LockDesc)
	PrintCmd( '<bar> show', L.ShowDesc)
	PrintCmd( '<bar> flashing', L.FlashingDesc)
	PrintCmd( '<bar> timer', L.TimerDesc)
	PrintCmd( '<bar> scale <value>', L.ScaleDesc)
	PrintCmd( '<bar> rows <value>', L.RowsDesc)
	PrintCmd( '<bar> cols <value>', L.ColsDesc)
	PrintCmd( '<bar> horizontal', L.HorizontalDesc)
	PrintCmd( '<bar> xPadding <value>', L.XPaddingDesc)
	PrintCmd( '<bar> yPadding <value>', L.YPaddingDesc)
	PrintCmd( '<bar> sort <value>', L.SortDesc)
	PrintCmd( 'version', L.VersionDesc)
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
local function GetProperty( info)
	local key = info[#info]
	return Module.profile[key]
end

local function SetProperty( info, value)
	local key = info[#info]
	Module.profile[key] = value
	Module:SendMessage( 'BISON_UPDATE')
end

local main = {
	type = 'group', order = 10, name = 'Main', get = GetProperty, set = SetProperty, handler = Addon, 
	args = {
		release     = { type = 'description', order = 20,  name = Addon.version, cmdHidden = true, fontSize = 'large' },
		description = { type = 'description', order = 30,  name = L.Description, cmdHidden = true, fontSize = 'large' },
		space1      = { type = 'description', order = 40,  name = ' ', cmdHidden = true },
		enabled     = { type = 'toggle',      order = 140, name = L.EnabledName, desc = L.EnabledDesc, get = 'IsEnabled',         set = SetEnable, width = 'full' },
		debug       = { type = 'toggle',      order = 150, name = L.DebugName,   desc = L.DebugDesc,   get = 'IsDebugLogEnabled', set = SetDebug,  width = 'full' },
		blizzard    = { type = 'toggle',      order = 160, name = L.BlizzName,   desc = L.BlizzDesc,   set = SetBlizzard, width = 'full' },
		locked      = { type = 'toggle',      order = 170, name = L.LockName,    desc = L.LockDesc,    width = 'full' },
	}
}
local plugins = {}
local options = {
	type = 'group', childGroups = 'tab', get = GetProperty, set = SetProperty, handler = Addon,
	args = {
		version  = { type = 'execute', order = 110, name = L.Version,     func = PrintVersion },
		help     = { type = 'execute', order = 120, name = L.Help,        func = PrintHelp },
		enabled  = { type = 'toggle',  order = 130, name = L.EnabledName, get = 'IsEnabled',         set = SetEnable },
		debug    = { type = 'toggle',  order = 140, name = L.DebugName,   get = 'IsDebugLogEnabled', set = SetDebug },
		blizzard = { type = 'toggle',  order = 150, name = L.BlizzName,   set = SetBlizzard },
		locked   = { type = 'toggle',  order = 160, name = L.LockName },
	},
	plugins = plugins,
}

--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Module:OnInitialize()
	self.profile = Addon.db.profile
	options.args.profile = LibStub( 'AceDBOptions-3.0'):GetOptionsTable( Addon.db)
	self:AddPlugin( 'bison_main', { main = main })
	local addonName = Addon:GetName()
	LibStub( 'AceConfig-3.0'):RegisterOptionsTable( addonName, options)
	LibStub( 'AceConfigDialog-3.0'):AddToBlizOptions( addonName, addonName, nil, 'main')
	Addon:RegisterChatCommand( 'bison', OnCommand)
	Addon:RegisterChatCommand( 'bi', OnCommand)
end

local addProfile = true
function Module:OnEnable()
	if addProfile then
		pcall( function() options.args.profile.args.desc.dialogHidden = true end)
		local addonName = Addon:GetName()
		LibStub( 'AceConfigDialog-3.0'):AddToBlizOptions( addonName, L.Profile, addonName, 'profile')
		addProfile = false
	end
	self:RegisterMessage( 'BISON_UPDATE')
end

function Module:OnDisable()
	self:UnregisterMessage( 'BISON_UPDATE')
end

function Module:AddPlugin( name, arr)
	if type( arr) == 'table' then
		plugins[name] = arr
	end
end

function Module:BISON_UPDATE( event, bar)
	self.profile = Addon.db.profile
end

