--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'prototype'

local Addon    = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L        = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')
local AceGUI   = LibStub( 'AceGUI-3.0')

local NOTHING = {}
local SORT_DESC = {
	[1] = L.SortNone,
	[2] = L.SortAlpha,
	[3] = L.SortRevert,
	[4] = L.SortInc,
	[5] = L.SortDec,
	[6] = L.SortDurationAsc,
	[7] = L.SortDurationDesc
}
local SORT_TYPES = {
	['none']   = 1,
	['alpha']  = 2,
	['revert'] = 3,
	['inc']    = 4,
	['dec']    = 5,
	['durationasc']  = 6,
	['durationdesc'] = 7,
}

--[[---------------------------------------------------------------------------------
LBF
------------------------------------------------------------------------------------]]
local LBF = LibStub( 'LibButtonFacade', true)

if LBF then
	local function LBF_OnSkin( self, skin, gloss, backdrop, group, button, colors)
		if group then
			local profile = self.db.profile[group]
			if profile then
				if type( profile.style) ~= 'table' then
					profile.style = {}
				end
				local style = profile.style
				style.skin     = skin
				style.gloss    = gloss
				style.backdrop = backdrop
				style.colors   = colors
			end
		end
	end

	LBF:RegisterSkinCallback( Addon:GetName(), LBF_OnSkin, Addon)
end

local function AddToLBF( group)
	if LBF then
		local profile = Addon.db.profile[group]
		if profile then
			local style = profile.style or NOTHING
			LBF:Group( Addon:GetName(), group):Skin( style.skin, style.gloss, style.backdrop, style.colors)
		end
	end
end

local function AddChieldToLBF( group, button)
	if LBF then
		LBF:Group( Addon:GetName(), group):AddButton( button.frame)
	end
end

local function RemoveChieldFromLBF( group, button)
	if LBF then
		LBF:Group( Addon:GetName(), group):RemoveButton( button.frame)
	end
end

--[[---------------------------------------------------------------------------------
Sorting
------------------------------------------------------------------------------------]]
local function SortNone( a, b)
	return a.id < b.id
end

local function SortNameAsc( a, b)
	if a.name and b.name then return a.name < b.name end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local function SortNameDesc( a, b)
	if a.name and b.name then return a.name > b.name end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local function SortTimeLeftAsc( a, b)
	if a.name and b.name then
		if a.timeLeft and b.timeLeft then return a.timeLeft < b.timeLeft end
		if a.timeLeft then return true end
		if b.timeLeft then return false end
		return a.name < b.name
	end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local function SortTimeLeftDesc( a, b)
	if a.name and b.name then
		if a.timeLeft and b.timeLeft then return a.timeLeft > b.timeLeft end
		if a.timeLeft then return false end
		if b.timeLeft then return true end
		return a.name < b.name
	end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local function SortDurationAsc( a, b)
	if a.name and b.name then
		if a.duration and b.duration then return a.duration < b.duration end
		if a.duration then return true end
		if b.duration then return false end
		return a.name < b.name
	end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local function SortDurationDesc( a, b)
	if a.name and b.name then
		if a.duration and b.duration then return a.duration > b.duration end
		if a.duration then return true end
		if b.duration then return false end
		return a.name < b.name
	end
	if a.name then return true end
	if b.name then return false end
	return a.id < b.id
end

local SORT = {
	['none']   = SortNone,
	['alpha']  = SortNameAsc,
	['revert'] = SortNameDesc,
	['inc']    = SortTimeLeftAsc,
	['dec']    = SortTimeLeftDesc,
	['durationasc']  = SortDurationAsc,
	['durationdesc'] = SortDurationDesc,
}

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function DurationString( seconds, timer)
	if not timer then
		return format( SecondsToTimeAbbrev( seconds))
	end
	if not seconds then
		return ''
	end
	local negative, time
	if seconds < 0 then 
		negative = '-'
		time = -seconds
	else
		negative = ''
		time = seconds
	end
	if time < 60 then 
		return format( '|cffffffff%s%ds|r', negative, time) 
	end
	local min = floor( time / 60)
	if time < 600 then 
		return format( '|cffffffff%s%d:%02d|r', negative, min, time % 60) 
	end
	if time < 3600 then 
		return format( '|cffffffff%s%dm|r', negative, min) 
	end
	return format( '|cffffffff%s%d:%02dh|r', negative, floor( min / 60), min % 60)
end

local function Bar_OnPosition( group, event, x, y)
	local profile = Addon.db.profile[group.moduleName]
	profile.xPos = x
	profile.yPos = y
end

local function CreateGroup( name, color)
	local group = AceGUI:Create( 'BisonGroup')
	group:SetLayout( 'BuffBar')
	group.moduleName = name
	group:SetGhostColor( color)
	group:SetCallback( 'OnPosition', Bar_OnPosition)
	return group
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
local prototype = {
	['OnInitialize'] = function( self)
		self.profile = Addon.db.profile[self:GetName()]
		self.aura = {}
		self.group = CreateGroup( self:GetName(), self.color)
		AddToLBF( self:GetName())
		if type( self.OnModuleInitialize) == 'function' then
			self:OnModuleInitialize()
		end
	end,
	
	['OnEnable'] = function( self)
		self:UpdateGroup()
		self:RegisterMessage( 'BISON_UPDATE')
		self:RegisterMessage( 'BISON_DEBUG')
		self:SetVisible( true)
		if type( self.OnModuleEnable) == 'function' then
			self:OnModuleEnable()
		end
	end,
	
	['OnDisable'] = function( self)
		self:SetVisible( false)
		self:UnregisterMessage( 'BISON_UPDATE')
		self:UnregisterMessage( 'BISON_DEBUG')
		if type( self.OnModuleDisable) == 'function' then
			self:OnModuleDisable()
		end
	end,

	['CloneAura'] = function( self, name)
		Addon:CloneAura( self.aura, name)
	end,

	['RegisterOptions'] = function( self, options)
		local name = self:GetName()
		Addon:GetModule( 'Options'):AddPlugin( 'bison_'..name, { [name] = options })
		local addonName = Addon:GetName()
		LibStub( 'AceConfigDialog-3.0'):AddToBlizOptions( addonName, options.name, addonName, name)
	end,

	['BISON_UPDATE'] = function( self, event, bar)
		self.profile = Addon.db.profile[self:GetName()]
		self:Debug( event, bar)
		if bar == nil or bar == self.moduleName then
			self:UpdateGroup()
			self:SetVisible( not (self.hideOnLock and Addon.db.profile.locked))
		end
	end,

	['BISON_DEBUG'] = function( self, event, debug)
		self:ToggleDebugLog( debug)
	end,

	['Renumber'] = function( self)
		local group = self.group
		local max = self.profile.number
		while #group.children > max do
			local child = tremove( group.children)
			RemoveChieldFromLBF( self:GetName(), child)
			AceGUI:Release( child)
		end
		local n = #group.children + 1
		for i = n,max do
			local child = self:CreateButton()
			child:Initialize( i, self.filter)
			group:AddChild( child)
			AddChieldToLBF( self:GetName(), child)
		end
		group:DoLayout()
	end,

	['Sort'] = function( self)
		table.sort( self.aura, self:GetSort())
	end,

	['UpdateGroup'] = function( self)
		self:Renumber()
		self.group:SetPos( self.profile.xPos, self.profile.yPos)
		self.group:SetLock( Addon.db.profile.locked)
		self:Sort()
		self:UpdateValues()
	end,

	['UpdateChild'] = function( self, child, aura)
		local profile = self.profile
		child:SetUserData( 'id',   aura.id)
		child:SetUserData( 'name', aura.name)
		child:SetText( aura.timeLeft and (SHOW_BUFF_DURATIONS == '1') and DurationString( aura.timeLeft, profile.timer) or nil, aura.warn)
		child:SetSpell( profile.spell and aura.spellID)
		child:SetCount( aura.count)
		child:SetTexture( aura.texture)
		if profile.flashing and aura.timeLeft then
			if aura.duration and aura.duration < BUFF_WARNING_TIME then
				child:SetFlashing( aura.timeLeft < aura.duration/2)
			else
				child:SetFlashing( aura.timeLeft < BUFF_WARNING_TIME)
			end
		else
			child:SetFlashing( false)
		end
		child:SetGap( profile.gap)
	end,

	['SetVisible'] = function( self, visible)
		if visible and self.profile.show then
			Addon:AddModule( self:GetName())
			self.group:SetVisible( true)
		else
			self.group:SetVisible( false)
			Addon:RemoveModule( self:GetName())
		end
	end,

	['IsLock'] = function( self)
		return Addon.db.profile.locked
	end,
	
	['GetSort'] = function( self)
		local sort = self.profile.sort
		return SORT[sort] or SortNone
	end,

	['GetConsolidation'] = function( self)
		return Addon:GetConsolidation()
	end,

	['GetProperty'] = function( self, info)
		local key = info[#info]
		return self.profile[key]
	end,
	
	['SetProperty'] = function( self, info, value)
		local key = info[#info]
		self.profile[key] = value
		self:SendMessage( 'BISON_UPDATE', self.moduleName)
	end,
	
	['SetScale'] = function( self, info, value)
		value = tonumber(value)
		if value and value > 0 and value <= 2 then
			self.profile.scale = value
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetCols'] = function( self, info, value)
		local maxButton = info.option.max or 16
		value = tonumber(value)
		if value and value > 0 and value <= maxButton then
			value = floor( value)
			self.profile.cols = value
			self.profile.rows = ceil( self.profile.number / value)
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetRows'] = function( self, info, value)
		local maxButton = info.option.max or 16
		value = tonumber(value)
		if value and value > 0 and value <= maxButton then
			value = floor( value)
			self.profile.cols = ceil( self.profile.number / value)
			self.profile.rows = value
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetNumber'] = function( self, info, value)
		local maxButton = info.option.max or 16
		value = tonumber(value)
		if value and value > 0 and value <= maxButton then
			value = floor( value)
			self.profile.number = floor( value)
			self.profile.cols   = ceil( self.profile.number / self.profile.rows)
			self.profile.rows   = ceil( self.profile.number / self.profile.cols)
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetXPadding'] = function( self, info, value)
		local min = info.option.mix or -20
		local max = info.option.max or 20
		value = tonumber(value)
		if value and value >= min and value <= max then
			self.profile.xPadding = floor( value)
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetYPadding'] = function( self, info, value)
		local min = info.option.mix or -50
		local max = info.option.max or 50
		value = tonumber(value)
		if value and value >= min and value <= max then
			self.profile.yPadding = floor( value)
			self:SendMessage( 'BISON_UPDATE', self.moduleName)
		end
	end,
	
	['SetGap'] = function( self, info, value)
		self.profile.gap = tonumber( value) or 0
		self:SendMessage( 'BISON_UPDATE', self.moduleName)
	end,
	
	['SetFlashing'] = function( self, info, value)
		self.profile.flashing = value
		self:SendMessage( 'BISON_UPDATE', self.moduleName)
	end,
	
	['GetJoin'] = function( self)
		return Addon.db.profile.join
	end,
	
	['SetJoin'] = function( self, info, value)
		Addon.db.profile.join = value
		self:SendMessage( 'BISON_UPDATE', self.moduleName)
	end,
	
	['GetSortDesc'] = function( self)
		return SORT_DESC
	end,
	
	['GetSortType'] = function( self)
		return SORT_TYPES[self.profile.sort]
	end,
	
	['SetSortType'] = function( self, info, value)
		for k,v in pairs(SORT_TYPES) do
			if v == value then
				self.profile.sort = k
				self:SendMessage( 'BISON_UPDATE', self.moduleName)
				break
			end
		end
	end
}
--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Addon:NewBarModule( name, ...)
	if self:GetModule( name, true) then
		error( 'can not register double module: '..name, 2)
	end
	return self:NewModule( name, prototype, 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0', ...)
end

